package controller;
import view.*;
import model.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * STUDENTS:  ADD CODE AT THE SPECIFIED LOCATIONS BELOW
 * This class represents the "controller" who instantiates the model and view plus the adapters
 * that hook them together.
 * The controller extends JApplet simply so that it can be run as an applet 
 * as well as a regular application.
 */
public class AppController extends JApplet{
  
  private MyModel _model;
  private MyView _view;
  private WindowListener _wl = new WindowAdapter(){}; //Need to extend the abstract, no-op WindowAdapter 
  
  /**
   * Constructor for controller.
   * Used when called as an application from main.
   * Calls init().
   * Uses a window listener that calls System.exit(0) for the frame.
   */
  public AppController(WindowListener wl) {
    _wl = wl;
    init();
  }
  
  /**
   * Constructor for the controller.  Called when used as an applet in a browser 
   * Browser will call init() automatically after construction.
   * Uses default no-op window listener for the frame so that it won't close 
   * down the browser when it closes.
   */
  public AppController() {
  }
  
  /**
   * Initializes the controller.
   * Constructs the model, view and the adapters to connect them.
   * STUDENTS: ADD CODE TO THIS METHOD (TWICE)
   */ 
  public void init(){
    //Instantiate the MyView object _view with a title for its frame, and an IModelAdapter
    // whose sole method calls the model's getQuoteFrom method with the supplied input.
    // STUDENT CODE STARTS HERE:
    _view = new MyView("The Quote-maker", _wl, new IModelAdapter() {
      public String getResponseTo(String inp) {
        return _model.getQuoteFrom(inp);
      } 
    });
    //END OF STUDENT CODE
    
    // Instantiate the MyModel object _model with a IViewAdapter whose one method will set the view's 
    // bottom label to the given string.
    // STUDENT CODE STARTS HERE:
    _model = new MyModel(new IViewAdapter() {
      public void display(String s) {
        _view.setBottomLabel(s);
      }
    });
    //END OF STUDENT CODE
  }
  
  /**
   * Runs the application/applet by showing the view
   * Called automatically by a browser when used as an applet.
   */
  public void start() {
    _view.setVisible(true);    
  }
  
  /**
   * Main method that constructs the controller and starts it.
   * Gives controller a window listener that calls System.exit(0)
   * when the view is closed.
   */
  public static void main(String[] args) {
    /**
     * Add an anonymous WindowAdapter event handler to call System.exit to
     * force termination of the main application when the Frame closes.
     * Without it, the main application will still be running even after
     * the frame is closed.
     */
    (new AppController(new java.awt.event.WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    })).start();
  }
}